#!/bin/sh
##
## bootstrap_dmx.sh - Bootsrap DMX Docker Container
##
## author: Juergen Neumann <juergen@dmx.berlin>
## (c) DMX Systems 2021 <https://dmx.systems>
##
## This program is free software: you can redistribute
## it and/or modify it under the terms of the 
## GNU General Public License as published by the 
## Free Software Foundation, either version 3 of 
## the License, or (at your option) any later version. 
##
##
DMX_DIR='/opt/dmx'

## Debug
echo "" > ${DMX_DIR}/logs/bootstrap.log
echo "*** DMX Docker Container started $( date ) ***" >> ${DMX_DIR}/logs/bootstrap.log
echo "WHOAMI=$( whoami )" >> ${DMX_DIR}/logs/bootstrap.log
echo "USER_NAME=${USER_NAME}" >> ${DMX_DIR}/logs/bootstrap.log
echo "USER_ID=${USER_ID}" >> ${DMX_DIR}/logs/bootstrap.log
echo "GROUP_ID=${GROUP_ID}" >> ${DMX_DIR}/logs/bootstrap.log
echo "DEBUG=${DEBUG}" >> ${DMX_DIR}/logs/bootstrap.log
echo "" >> ${DMX_DIR}/logs/bootstrap.log

## Functions

function log_msg() {
    echo "$( date +%T.%2N ) $1" # The logging output
    echo "$( date +%T.%2N ) $1" >> ${DMX_DIR}/logs/bootstrap.log
}

function create_user() {
    ## Creates the user that runs DMX inside the container
    ## All variables here are global environment vars
    if [ -z "$( getent passwd ${USER_NAME} | grep "${USER_ID}" | grep "${GROUP_ID}" )" ]; then
        log_msg "INFO: Creating user ${USER_NAME} with uid=${USER_ID} and gid=${GROUP_ID}."
        addgroup -g ${GROUP_ID} ${USER_NAME}
        adduser -h ${DMX_DIR} -H -D -u ${USER_ID} -G ${USER_NAME} -s /bin/bash ${USER_NAME}
    else
        log_msg "INFO: Found user ${USER_NAME} with uid=${USER_ID} and gid=${GROUP_ID}."
    fi
}

function run_commands() {
    ## TODO: add check if commands need to run again
    ## check if there are any files in run.d
    IFS=$'\n'
    local command_files="$( find ${DMX_DIR}/run.d/ -type f | sort -n )"
    # log_msg "INFO: Found ${command_files}"
    if [ ! -z "${command_files}" ]; then
        # log_msg "INFO: Found ${command_files}"
        for command_file in ${command_files}; do
            log_msg "INFO: Processing ${command_file}"
            while IFS= read -r line; do
                log_msg "INFO: Running ${line}"
                /bin/sh -c "${line}" | tee -a ${DMX_DIR}/logs/bootstrap.log 2>&1
            done < ${command_file}
        done
    fi
}

function file_changed() {
    ## Checks if a file was changed via md5sum
    ## Returns '${file_name}: OK' if the file was not changed
    IFS=$'\n'
    local file_name="$1"
    local file_basename="$( basename "${file_name}" )"
    local hashes_dir="${DMX_DIR}/logs/.md5hashes"
    if [ ! -d ${hashes_dir} ]; then
        mkdir -p ${hashes_dir}
        chown -R ${USER_ID}:${GROUP_ID} ${hashes_dir}
    fi
    local md5_list="${hashes_dir}/._bootstrap_md5sum_${file_basename}"
    if [ ! -f ${md5_list} ]; then
        touch ${md5_list}
        chown ${USER_ID}:${GROUP_ID} ${md5_list}
    else
        ## check all files still exist
        while read existing_entry; do
            this_file="$( echo "${existing_entry}" | cut -d' ' -f2- | sed 's/\ //g' )"
            if [ ! -f ${this_file} ]; then
                sed -i "\|${this_file}|d" ${md5_list}
            fi
        done < ${md5_list}
    fi
    ##
    if [ "$( grep -w "${file_name}" ${md5_list} )" ]; then
        local result="$( md5sum -c ${md5_list} 2>/dev/null | grep ${file_name}: )"
        ## Update if result not OK
        if [ -z "$( echo "${result}" | grep "OK" )" ]; then
            sed -i "\|${file_name}|d" ${md5_list}
            md5sum ${file_name} >> ${md5_list}
            echo "${file_name}: UPDATE"
        else
            echo "${result}"
        fi
    else
        md5sum ${file_name} >> ${md5_list}
        echo "${file_name}: NEW"
    fi
}

function config_properties() {
    ## Modifies config files if config snippets
    ## are provided in config.properties.d or
    ## logging.properties.d volumes.
    IFS=$'\n'
    dmx_conf_dir="${DMX_DIR}/conf"
    conf_d_dir="${DMX_DIR}/conf.d"
    work_dir="${DMX_DIR}/bootstrap/tmp"
    if [ ! -d ${work_dir} ]; then
        mkdir ${work_dir}
    fi
    cp_d_dir="${work_dir}/config.properties.d"
    lp_d_dir="${work_dir}/logging.properties.d"
    if [ ! -d ${cp_d_dir} ]; then
        mkdir ${cp_d_dir}
        chown -R ${USER_ID}:${GROUP_ID} ${cp_d_dir}
    fi
    if [ ! -d ${lp_d_dir} ]; then
        mkdir ${lp_d_dir}
        chown -R ${USER_ID}:${GROUP_ID} ${lp_d_dir}
    fi

    ## Only copy *.properties files from bootstrap to dmx_conf_dir,
    ## if they do not already exist.
    if [ ! -f ${dmx_conf_dir}/config.properties ]; then
        log_msg "INFO: Bootstrap copying ${DMX_DIR}/bootstrap/config.properties to ${dmx_conf_dir}/"
        cp ${DMX_DIR}/bootstrap/config.properties ${dmx_conf_dir}/
        chown ${USER_ID}:${GROUP_ID} ${dmx_conf_dir}/config.properties
        ## make sure to remove old hashes, when starting with fresh config
        if [ -d ${DMX_DIR}/logs/.md5hashes ]; then
            log_msg "INFO: Removing old file hashes."
            rm -rf ${DMX_DIR}/logs/.md5hashes
        fi
    fi
    if [ ! -f ${dmx_conf_dir}/logging.properties ]; then
        log_msg "INFO: Bootstrap copying ${DMX_DIR}/bootstrap/logging.properties to ${dmx_conf_dir}/"
        cp ${DMX_DIR}/bootstrap/logging.properties ${dmx_conf_dir}/
        chown ${USER_ID}:${GROUP_ID} ${dmx_conf_dir}/logging.properties
        ## make sure to remove old hashes, when starting with fresh config
        if [ -d ${DMX_DIR}/logs/.md5hashes ]; then
            log_msg "INFO: Removing old file hashes."
            rm -rf ${DMX_DIR}/logs/.md5hashes
        fi
    fi

    ## Only add bootstrap defaults, if they were not already applied
    ## (check for existing hash files)
    if [ -f ${DMX_DIR}/bootstrap/0000_config_properties_bootstrap ]; then
        result="$( file_changed "${DMX_DIR}/bootstrap/0000_config_properties_bootstrap" )"
        log_msg "INFO: Status ${result}"
        if [ -z "$( echo "${result}" | grep "OK" )" ]; then
            log_msg "INFO: Bootstrap copying ${DMX_DIR}/bootstrap/0000_config_properties_bootstrap to ${cp_d_dir}/"
            cp ${DMX_DIR}/bootstrap/0000_config_properties_bootstrap ${cp_d_dir}/
        fi
    else
        log_msg "ERROR! File ${DMX_DIR}/bootstrap/0000_config_properties_bootstrap not found."
        exit 1
    fi
    ##
    if [ -f ${DMX_DIR}/bootstrap/0000_logging_properties_bootstrap ]; then
        result="$( file_changed "${DMX_DIR}/bootstrap/0000_logging_properties_bootstrap" )"
        log_msg "INFO: Status ${result}"
        if [ -z "$( echo "${result}" | grep "OK" )" ]; then
            log_msg "INFO: Bootstrap copying ${DMX_DIR}/bootstrap/0000_logging_properties_bootstrap to ${lp_d_dir}/"
            cp ${DMX_DIR}/bootstrap/0000_logging_properties_bootstrap ${lp_d_dir}/
        fi
    else
        log_msg "ERROR! File ${DMX_DIR}/bootstrap/0000_logging_properties_bootstrap not found."
        exit 1
    fi
    ##

    ## Check config snippets and copy them
    if [ -d ${conf_d_dir}/config.properties.d ]; then
        cp_snippets="$( find ${conf_d_dir}/config.properties.d/ -type f | sort -n )"
    else
        cp_snippets=''
    fi
    for snippet in ${cp_snippets}; do
        log_msg "INFO: Analyzing ${snippet}."
        result="$( file_changed "${snippet}" )"
        log_msg "INFO: Status ${result}"
        if [ -z "$( echo "${result}" | grep "OK" )" ]; then
            log_msg "INFO: Found new or changed config snippet ${snippet}."
            cp ${snippet} ${cp_d_dir}/
        fi
    done
    if [ -d ${conf_d_dir}/logging.properties.d ]; then
        lp_snippets="$( find ${conf_d_dir}/logging.properties.d/ -type f | sort -n )"
    else
        lp_snippets=''
    fi
    for snippet in ${lp_snippets}; do
        log_msg "INFO: Analyzing ${snippet}."
        result="$( file_changed "${snippet}" )"
        log_msg "INFO: Status ${result}"
        if [ -z "$( echo "${result}" | grep "OK" )" ]; then
            log_msg "INFO: Found new or changed config snippet ${snippet}."
            cp ${snippet} ${lp_d_dir}/
        fi
    done

    ## Check if changes are provided via config snippets in $cp_d_dir
    if [ ! -z "$(ls -A ${cp_d_dir}/)" ]; then
        log_msg "INFO: New config.properties snippets require verification."
        ## Get recent copy of config properties
        cp ${dmx_conf_dir}/config.properties ${work_dir}/
    fi
    ## Check if changes are provided via config snippets in $lp_d_dir
    if [ ! -z "$(ls -A ${lp_d_dir}/)" ]; then
        log_msg "INFO: New logging.properties snippets require verification."
        ## Get recent copy of logging properties
        cp ${dmx_conf_dir}/logging.properties ${work_dir}/
    fi

    ## Configure files
    config_files="$( find ${work_dir}/ -type f -name "*.properties" | sort -n )"
    for conf_file in ${config_files}; do
        file_name="$( basename ${conf_file} )"
        log_msg "INFO: Analyzing settings for ${file_name}."
        config_parts="$( find ${work_dir}/${file_name}.d/ -type f | sort -n )"
        prop_file="${conf_file}_changes"
        ## Ensure file does not exist
        if [ -f ${prop_file} ]; then
            rm ${prop_file}
        fi
        ## Collect changes
        touch ${prop_file}
        for part in ${config_parts}; do
            log_msg "INFO: Collecting config snippet ${part}."
            cat ${part} | grep -v '^#' | grep -v '^$' >>${prop_file}
        done
        ## Apply changes, if required
        if [ -f ${prop_file} ]; then
            log_msg "INFO: Suggested changes for ${file_name}:"
            log_msg "$( cat ${prop_file} )"
            cp ${conf_file} ${conf_file}.new
            while read property; do
                ## get key and remove all leading and trailing spaces
                key="$( echo "${property}" | cut -d'=' -f1 | sed 's/^[[:space:]]*//' | sed 's/[[:space:]]*$//' )"
                ## Escape ampersand ('&') in value for sed command ('&' is often used in LDAP Filter) and remove leading and trailing '"'.
                value="$( echo "${property//&/\\&}" | cut -d'=' -f2- | sed 's/^[[:space:]]*//' | sed 's/[[:space:]]*$//' | sed 's/^\"//' | sed 's/\"$//' )"
                ## Analyze commands
                if [ ! -z "$( echo "${key}" | grep '^+#' )" ]; then
                    ## Comment out the line
                    key="$( echo "${key}" | sed 's/^+#//' )"
                    if [ ! -z "$( cat ${conf_file}.new | grep "^${key}[[:space:]]=" | grep "=[[:space:]]${value}[[:space:]]$" )" ]; then
                        log_msg "INFO: Found new config ${property}. Commenting out."
                        sed -i "s|${key}.*=.*${value}|# ${key} = ${value}|" ${conf_file}.new
                    fi
                elif [ ! -z "$( echo "${key}" | grep '^-#' )" ]; then
                    ## Remove comment from the line
                    key="$( echo ${key} | sed 's/^-#//' )"
                    if [ ! -z "$( cat ${conf_file}.new | grep "^${key}[[:space:]]=" | grep "=[[:space:]]${value}[[:space:]]$" )" ]; then
                        log_msg "INFO: Found new config ${property}. Commenting in."
                        sed -i "s|#.*${key}.*=.*${value}|${key} = ${value}|" ${conf_file}.new
                    fi
                elif [ ! -z "$( cat ${conf_file}.new | grep -v '^#' | grep "^${key}[[:space:]]=" | grep -v "=[[:space:]]${value}[[:space:]]$" )" ]; then
                    ## Change existing value
                    log_msg "INFO: Found new config ${key} = ${value}. Changing existing value."
                    sed -i "s|^${key}.*=.*|${key} = ${value}|" ${conf_file}.new
                elif [ -z "$( cat ${conf_file}.new | grep "^${key}[[:space:]]=" )" ]; then
                    ## Add new key and value
                    log_msg "INFO: Found new config ${key} = ${value}. Adding new property."
                    echo "" >> ${conf_file}.new
                    echo "## ${key} added via bootstrap" >> ${conf_file}.new
                    echo "${key} = ${value}" >> ${conf_file}.new
                else
                    log_msg "INFO: No changes required for ${key}."
                fi
            done < ${prop_file}

            ## Replace variable property values with their values from environment
            echo "$( envsubst <${conf_file}.new )" >${conf_file}.new

            ## Check for diff and update dmx config if required
            if [ "$( diff ${dmx_conf_dir}/${file_name} ${conf_file}.new )" != "" ]; then
                log_msg "INFO: Applying diffs for ${file_name}."
                mv ${conf_file}.new ${dmx_conf_dir}/${file_name}
                chown ${USER_ID}:${GROUP_ID} ${dmx_conf_dir}/${file_name}
            else
                log_msg "INFO: No diffs for ${file_name} found."
                rm ${conf_file}.new
            fi
            rm ${prop_file}
            rm ${conf_file}
        fi
    done
    ## Clean up
    if [ -d ${work_dir} ]; then
        rm -rf ${work_dir}
    fi
}

function read_property() {
    this_property="$1"
    ## We convert the java properties into shell vars via
    ## awk first, since they might include multiline values.
    CONFIG_PROPERTIES="$( cat ${DMX_DIR}/conf/config.properties | awk -f ${DMX_DIR}/bootstrap/read_properties.awk | grep -v ^# )"
    property="$( echo "${this_property}" | sed s'/\_/__/g' | sed 's/\./_/g' | tr [:lower:] [:upper:] )"
    value="$( echo "${CONFIG_PROPERTIES}" | grep ${property} | cut -d'=' -f2 )"
    echo "${value}"
}


function copy_jars() {
    ## Copies plugin jars from plugins to bundle-deploy folder
    ## Updates to newer versions of a plugin, if old one exists
    IFS=$'\n'
    local jars="$( find ${DMX_DIR}/plugins/ -type f -name "*.jar" )"
    for jar in ${jars}; do
        log_msg "INFO: Checking plugin jar ${jar}"
        local file_name="$(basename -- "${jar}")"
        local plugin="$(echo "${file_name%.*}" \
            | sed 's/\-latest//g' \
            | sed -e 's/\-[0-9].*$//g' \
            | sed 's/\-v$//g' \
            | sed 's/\.v$//g' \
            | sed 's/\-$//g' \
            | sed 's/\.$//g')"
        ## find and delete older versions of jars in bundle-deploy
        local old_jars="$(find ${DMX_DIR}/bundle-deploy/ -type f -name "${plugin}-[0-9]*.jar")"
        ## make sure new jar get installed, if it does no exist.
        local copy_jar="YES"
        for old_jar in ${old_jars}; do
            if [ "$(basename ${old_jar})" != "$(basename ${jar})" ]; then
                log_msg "INFO: Removing exisiting file ${old_jar}."
                rm "${old_jar}"
            else
                result="$( file_changed "${jar}" )"
                #log_msg "INFO: Status ${result}"
                if [ -z "$( echo "${result}" | grep "OK" )" ]; then
                    log_msg "INFO: Found new or changed plugin ${jar}."
                    log_msg "INFO: Removing exisiting file ${old_jar}."
                    rm "${old_jar}"
                else
                    log_msg "INFO: Skipping installation of ${jar}. File exists."
                    ## skip installation of existing jar file
                    local copy_jar="NO"
                fi
            fi
        done
        if [ "${copy_jar}" == "YES" ]; then
            ## test for various versions of same plugin
            local count_jars="$( find ${DMX_DIR}/plugins/ -type f -name "${plugin}-[0-9]*.jar" | grep -v "${plugin}*.[a-zA-Z].*.jar$" | wc -l )"
            if [ ${count_jars} -eq 1 ]; then
                log_msg "INFO: Installing ${file_name}."
                ## make sure we reset the bundle cache
                rm -rf ${DMX_DIR}/bundle-cache
                ## register file
                file_changed "${jar}"
                cp ${jar} ${DMX_DIR}/bundle-deploy/
                chown ${USER_ID}:${GROUP_ID} ${DMX_DIR}/bundle-deploy/${file_name}
            else
                log_msg "ERROR! Found ${count_jars} conflicting versions of ${plugin} to be installed."
                log_msg "WARNING! Skipping installation of ${jar}"
            fi
        fi
    done
}


function copy_modules() {
    ## Copies platform related module jars from modules to bundle folder.
    ## Overwrites any module with staged version of changed module jar.
    IFS=$'\n'
    local jars="$( find ${DMX_DIR}/modules/ -type f -name "*.jar" )"
    for jar in ${jars}; do
        log_msg "INFO: Checking module jar ${jar}"
        local file_name="$(basename -- "${jar}")"
        local module="$(echo "${file_name%.*}" \
            | sed 's/\-latest//g' \
            | sed -e 's/\-[0-9].*$//g' \
            | sed 's/\-v$//g' \
            | sed 's/\.v$//g' \
            | sed 's/\-$//g' \
            | sed 's/\.$//g')"
        ## find and delete older versions of jars in bundle folder
        local old_jars="$(find ${DMX_DIR}/bundle/ -type f -name "${module}-[0-9]*.jar")"
        ## make sure new jar get installed, if it does no exist.
        local copy_jar="YES"
        for old_jar in ${old_jars}; do
            if [ "$(basename ${old_jar})" != "$(basename ${jar})" ]; then
                log_msg "INFO: Removing exisiting file ${old_jar}."
                rm "${old_jar}"
            else
                result="$( file_changed "${jar}" )"
                #log_msg "INFO: Status ${result}"
                if [ -z "$( echo "${result}" | grep "OK" )" ]; then
                    log_msg "INFO: Found new or changed module ${jar}."
                    log_msg "INFO: Removing exisiting file ${old_jar}."
                    rm "${old_jar}"
                else
                    log_msg "INFO: Skipping installation of ${jar}. File exists."
                    ## skip installation of existing jar file
                    local copy_jar="NO"
                fi
            fi
        done
        if [ "${copy_jar}" == "YES" ]; then
            ## test for various versions of same plugin
            local count_jars="$( find ${DMX_DIR}/modules/ -type f -name "${module}-[0-9]*.jar" | grep -v "${module}*.[a-zA-Z].*.jar$" | wc -l )"
            if [ ${count_jars} -eq 1 ]; then
                log_msg "INFO: Installing ${file_name}."
                ## make sure we reset the bundle cache
                rm -rf ${DMX_DIR}/bundle-cache
                ## register file
                file_changed "${jar}"
                cp ${jar} ${DMX_DIR}/bundle/
                chown ${USER_ID}:${GROUP_ID} ${DMX_DIR}/bundle/${file_name}
            else
                log_msg "ERROR! Found ${count_jars} conflicting versions of ${module} to be installed."
                log_msg "WARNING! Skipping installation of ${jar}"
            fi
        fi
    done
}


function import_data() {
    ## Import data from dmx-filedir.tar.gz and dmx-db.tar.gz
    if [ -f ${DMX_DIR}/import/dmx-filedir.tar.gz ]; then
        FILEREPO="$( read_property dmx.filerepo.path )"
        if [ "${FILEREPO:0:1}" != "/" ]; then
            FILEREPO=${DMX_DIR}/${FILEREPO}
        fi
        REPOPATH="$( echo "${FILEREPO}" | sed 's/\/dmx-filedir//g' )"
        log_msg "INFO: Found file ${DMX_DIR}/import/dmx-filedir.tar.gz."
        log_msg "INFO: Importing files to $REPOPATH/dmx-filedir."
        rm -rf ${FILEREPO}/*
        tar xzf ${DMX_DIR}/import/dmx-filedir.tar.gz --directory=${REPOPATH}
    fi
    if [ -f ${DMX_DIR}/import/dmx-db.tar.gz ]; then
        DBDIR="$( read_property dmx.database.path )"
        if [ "${DBDIR:0:1}" != "/" ]; then
            DBDIR=${DMX_DIR}/${DBDIR}
        fi
        DBPATH="$( echo "${DBDIR}" | sed 's/\/dmx-db//g' )"
        log_msg "INFO: Found file ${DMX_DIR}/import/dmx-db.tar.gz."
        log_msg "INFO: Importing database to ${DBPATH}/dmx-db."
        rm -rf ${DBDIR}/*
        tar xzf ${DMX_DIR}/import/dmx-db.tar.gz --directory=${DBPATH}
    fi
}

function copy_extra_config_files() {
    dmx_conf_dir="${DMX_DIR}/conf"
    conf_d_dir="${DMX_DIR}/conf.d"
    if [ -d ${conf_d_dir} ]; then
        local this_list="$( find ${conf_d_dir}/* -maxdepth 0 ! -name *.properties.d )"
    else
        local this_list=""
    fi
    local result=''
    for this in ${this_list}; do
        base_name="$( basename ${this} )"
        if [ -f ${this} ]; then
            result="$( file_changed ${this} )"
            log_msg "INFO: Status ${result}"
            if [ -z "$( echo "${this}" | grep "${sub_dir}" )" ]; then
                sub_dir='/'
            fi
        elif [ -d ${this} ]; then
            if [ ! -d ${dmx_conf_dir}/${base_name} ]; then
                log_msg "INFO: Creating directory ${dmx_conf_dir}/${base_name}."
                mkdir ${dmx_conf_dir}/${base_name}
                chown -R ${USER_ID}:${GROUP_ID} ${dmx_conf_dir}/${base_name}
            fi
            sub_dir="/${base_name}/"
        fi
        if [ ! -d ${this} ] && [ -z "$( echo "${result}" | grep "OK" )" ]; then
            log_msg "INFO: Copying ${this} to ${dmx_conf_dir}${sub_dir}."
            cp ${this} ${dmx_conf_dir}${sub_dir}
            chown ${USER_ID}:${GROUP_ID} ${dmx_conf_dir}${sub_dir}/${base_name}
        elif [ -d ${this} ] && [ -z "$( echo "${result}" | grep "OK" )" ]; then
            log_msg "INFO: Copying ${this} to ${dmx_conf_dir}${sub_dir}."
            cp -a ${this} ${dmx_conf_dir}
            chown ${USER_ID}:${GROUP_ID} ${dmx_conf_dir}${sub_dir}/${base_name}
        fi
        result=''
    done
}

function set_permissions() {
    ## Sets the permissions for DMX folders
    IFS=' '
    log_msg "INFO: Checking ownerships for DMX dirs."
    chmod 644 LICENSE.txt
    chmod 644 COPYRIGHT.txt
    chown ${USER_ID}:${GROUP_ID} ${DMX_DIR}
    dmx_dirs="conf logs dmx-db dmx-filedir bundle-deploy bundle-available bundle-cache conf.d plugins modules run.d"
    ro_mounts="$( cat /proc/mounts | grep "[[:space:]]ro[[:space:],]" )"
    for this_dir in ${dmx_dirs}; do
        if [ -n "$( echo ${ro_mounts} | grep " ${DMX_DIR}/${this_dir} " )" ]; then
            log_msg "INFO: Directory ${DMX_DIR}/${this_dir} is mounted read-only."
        else
            if [ ! -d ${DMX_DIR}/${this_dir} ]; then
                mkdir ${DMX_DIR}/${this_dir}
            fi
            if [ "$( stat -c '%u:%g' ${DMX_DIR}/${this_dir} )" != "${USER_ID}:${GROUP_ID}" ]; then
                log_msg "INFO: Running chown -R ${USER_ID}:${GROUP_ID} ${DMX_DIR}/${this_dir}."
                chown -R ${USER_ID}:${GROUP_ID} ${DMX_DIR}/${this_dir}
            fi
        fi
    done
    IFS=$'\n'
}

function set_debugging() {
    ## Turns the Java debugging on or off
    if [ -n "${DEBUG}" ]; then
        log_msg "INFO: Debugging is enabled."
        DEBUG_OPTS='-agentlib:jdwp=server=y,transport=dt_socket,address=8000,suspend=y'
    else
        log_msg "INFO: Debugging is disabled."
        DEBUG_OPTS=
    fi
}

## run DMX
create_user
run_commands
config_properties
copy_jars
copy_modules
import_data
copy_extra_config_files
set_permissions
set_debugging
java_version="$( java -version 2>&1 | grep openjdk | cut -d'"' -f2 )"
log_msg "INFO: Starting Java VM ${java_version} (Xms=${XMS}, Xmx=${XMX})"
chown ${USER_NAME}:${USER_NAME} ${DMX_DIR}/logs/bootstrap.log
exec su -s /bin/sh -c "/usr/bin/java ${DEBUG_OPTS} -Xms${XMS} -Xmx${XMX} -Dfile.encoding=UTF-8 \
    -Dfelix.system.properties=file:${dmx_conf_dir}/config.properties \
    -jar ${DMX_DIR}/bin/felix.jar" ${USER_NAME}
